/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

( function ( $ ) {

    "use strict";

    /**
     * @param duDialog
     * @param window.mdpStellar.ajaxURL
     * @param window.mdpStellar.resetDialog - Reset dialog message
     * @param window.mdpStellar.importDialog - Import dialog message
     */
    $( document ).ready( function () {

        const mdpStellar = window.mdpStellar;

        /**
         * Show/hide Text by Select value
         * @param el
         */
        function hideTextBySelect ( el ) {

            let element = $( el );
            function showEl() {

                if ( element.val() !== 'hide' ) {
                    element.closest( 'tr' ).next().show( 300 );
                } else {
                    element.closest( 'tr' ).next().hide( 300 );
                }

            }
            element.on( 'change', showEl );
            showEl();

        }

        hideTextBySelect( '#mdp_stellar_backend_settings_latest' );
        hideTextBySelect( '#mdp_stellar_backend_settings_results' );
        hideTextBySelect( '#mdp_stellar_backend_settings_votes' );

        /**
         * Show/hide Advanced Schema Markup Textarea.
         **/
        let jsonLDSwitcher = $( '#mdp_stellar_schema_settings_advanced_markup' );
        function ShowJsonLDField() {

            if ( jsonLDSwitcher.prop( 'checked' ) === true ) {
                jsonLDSwitcher.closest( 'tr' ).next().show( 300 );
                jsonLDSwitcher.closest( 'tr' ).next().next().show( 300 );
            } else {
                jsonLDSwitcher.closest( 'tr' ).next().hide( 300 );
                jsonLDSwitcher.closest( 'tr' ).next().next().hide( 300 );
            }
        }
        jsonLDSwitcher.on( 'click', ShowJsonLDField );
        ShowJsonLDField();

        /**
         * Show/hide Advanced Schema Markup Switcher.
         **/
        let markupSwitcher = $( '#mdp_stellar_schema_settings_google_search_results' );
        function ShowAdvancedMarkupField() {

            if ( markupSwitcher.prop( 'checked' ) === true ) {

                markupSwitcher.closest( 'tr' ).next().show( 300 );
                ShowJsonLDField();

            } else {

                markupSwitcher.closest( 'tr' ).next().hide( 300 ).next().hide( 300 );

            }

        }
        markupSwitcher.on( 'click', ShowAdvancedMarkupField );
        ShowAdvancedMarkupField();

        /** Shortcode switcher settings */
        const tab = 'mdp_stellar_shortcode_settings';

        /** Show/hide setting by switcher **/
        function displayBySwitcher( switcherId, settingId ) {
            $( switcherId ).prop( 'checked' ) ?
                $( settingId ).closest( 'tr' ).show( 300 ) :
                $( settingId ).closest( 'tr' ).hide( 300 );
        }

        /** Show/Hide for multiple options */
        function displayBySwitcher2( switcherId, switcherId2, settingId ) {
            $( switcherId ).prop( 'checked' ) && $( switcherId2 ).prop( 'checked' ) ?
                $( settingId ).closest( 'tr' ).show( 300 ) :
                $( settingId ).closest( 'tr' ).hide( 300 );
        }

        // Image
        $( `#${ tab }_top_image` ).on( 'click', function () {

            if ( $( `#${ tab }_top_image` ).prop( 'checked' ) ) {

                $( `#${ tab }_top_image_size` ).closest( 'tr' ).show( 300 );
                $( `#${ tab }_top_equal` ).closest( 'tr' ).show( 300 );
                if ( $( `#${ tab }_top_equal` ).prop( 'checked' ) ) { $( `#${ tab }_top_height` ).closest( 'tr' ).show( 300 ); }

            } else {

                $( `#${ tab }_top_image_size` ).closest( 'tr' ).hide( 300 );
                $( `#${ tab }_top_equal` ).closest( 'tr' ).hide( 300 );
                $( `#${ tab }_top_height` ).closest( 'tr' ).hide( 300 );
            }

        } );

        // Equal height
        $( `#${ tab }_top_equal` ).on( 'click', function () {

            if ( $( `#${ tab }_top_equal` ).prop( 'checked' ) ) {

                $( `#${ tab }_top_height` ).closest( 'tr' ).show( 300 );

            } else {

                $( `#${ tab }_top_height` ).closest( 'tr' ).hide( 300 );
            }

        } );

        // Title
        $( `#${ tab }_top_title` ).on( 'click', function () {


            $( `#${ tab }_top_title` ).prop( 'checked' ) ?
                $( `#${ tab }_top_title_tag` ).closest( 'tr' ).show( 300 ) :
                $( `#${ tab }_top_title_tag` ).closest( 'tr' ).hide( 300 );

        } );

        // Rating
        $( `#${ tab }_top_rating` ).on( 'click', function () {

            $( `#${ tab }_top_rating` ).prop( 'checked' ) ?
                $( `#${ tab }_top_size` ).closest( 'tr' ).show( 300 ) :
                $( `#${ tab }_top_size` ).closest( 'tr' ).hide( 300 );

        } );

        displayBySwitcher( `#${ tab }_top_image`, `#${ tab }_top_image_size` );
        displayBySwitcher( `#${ tab }_top_image`, `#${ tab }_top_equal` );
        displayBySwitcher( `#${ tab }_top_image`, `#${ tab }_top_height` );
        displayBySwitcher( `#${ tab }_top_title`, `#${ tab }_top_title_tag` );
        displayBySwitcher( `#${ tab }_top_rating`, `#${ tab }_top_size` );

        displayBySwitcher2( `#${ tab }_top_image`, `#${ tab }_top_equal`, `#${ tab }_top_height` );

    } );

    /**
     * Reset Stellar results.
     **/
    $( '#reset' ).on( 'click', function( e ) {
        e.preventDefault();

        new duDialog( 'Reset Stellar', mdpStellar.messages.reset, duDialog.OK_CANCEL, // jshint ignore:line
            {
                okText: 'Okay',
                callbacks: {
                    okClick: function() {

                        /** Disable button and show process. */
                        $( '#reset' ).attr( 'disabled', true ).addClass( 'mdp-spin' ).find( '.material-icons' ).text('refresh');

                        /** Prepare data for AJAX request. */
                        let data = {
                            action: 'reset_stellar',
                            nonce: mdpStellar.nonce,
                            doReset: 1
                        };

                        /** Make POST AJAX request. */
                        jQuery.post( mdpStellar.ajaxURL, data, function( response ) {

                            /** Show Error message if returned false. */
                            if ( response ) {
                                new duDialog( 'Error', mdpStellar.messages.error );
                            } else {
                                new duDialog( 'Success', mdpStellar.messages.cleared );
                            }

                        }, 'json' ).fail(function( response ) {

                            /** Show Error message if returned some data. */
                            console.log( response );
                            new duDialog( 'Reset Stellar: Error', mdpStellar.messages.error );

                        } ).always(function() {
                            /** Enable button again. */
                            $( '#reset' ).attr( 'disabled', false ).removeClass( 'mdp-spin' ).find( '.material-icons' ).text('close');
                        } );

                        this.hide();  // hides the dialog
                    }
                }

            });
    } );

    /** Copy from Rating Form */
    let buttonID = '#import-rating-form';
    $( buttonID ).on( 'click', function( e ) {
        e.preventDefault();

        new duDialog( 'Import rating from Rating Form', mdpStellar.messages.import, duDialog.OK_CANCEL, // jshint ignore:line
            {
                okText: 'Okay',
                callbacks: {
                    okClick: function() {

                        /** Disable button and show process. */
                        $( buttonID ).attr( 'disabled', true ).addClass( 'mdp-spin' ).find( '.material-icons' ).text('refresh');

                        /** Prepare data for AJAX request. */
                        let data = {
                            action: 'import_rating_form',
                            nonce: mdpStellar.nonce,
                        };

                        /** Make POST AJAX request. */
                        jQuery.post( mdpStellar.ajaxURL, data, function( response ) {

                            /** Show Error message if returned false. */
                            if ( response.status ) {
                                new duDialog( 'Success', `${ response.counter } ${ mdpStellar.messages.imported }` );
                            } else {
                                new duDialog( 'Error', mdpStellar.messages.error );
                            }

                        }, 'json' ).fail( function( response ) {

                            /** Show Error message if returned some data. */
                            console.warn( response );
                            new duDialog( 'Error', mdpStellar.messages.error );

                        } ).always( function() {

                            /** Enable button again. */
                            $( buttonID ).attr( 'disabled', false ).removeClass( 'mdp-spin' ).find( '.material-icons' ).text('close');

                        } );

                        this.hide();

                    }
                }
            });
    } );

} ( jQuery ) );
