<?php
/**
 * Stellar
 * Star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

use Merkulove\Stellar\Unity\Settings;
use Merkulove\Stellar\Unity\TabAssignments;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

final class AdminPostTable {

	/**
	 * The one true AdminPostTable.
	 *
	 * @since 1.0.0
	 * @access private
	 * @var AdminPostTable
	 **/
	private static $instance;

	private function __construct() {

		$options = Settings::get_instance()->options;
		if ( 'hide' === $options[ 'results_admin' ] ) { return; }

		/** Add Stellar column to Selected Posts list. */
        foreach ( $options['cpt_support'] as $cpt ) {

	        add_filter( "manage_{$cpt}_posts_columns", [$this, 'add_head_column'], 10 );
	        add_action( "manage_{$cpt}_posts_custom_column", [$this, 'add_content_column'], 10, 2 );

	        /** Register our custom column stellar-results as 'sortable'. */
	        add_filter( "manage_edit-{$cpt}_sortable_columns", [$this, 'sortable_result_column' ] );

        }

	}

	/**
	 * Add CONTENT for stellar column with results.
	 *
	 * @param string $column_name
	 * @param $post_ID
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 **/
	public function add_content_column( $column_name, $post_ID ) {

		$options = Settings::get_instance()->options;

		/** Work only with Stellar column. */
		if ( $column_name !== 'stellar-results' ) { return; }

		$get_voters = PluginLogic::get_instance()->number_of_voters($post_ID) ?? 0;
		$get_rating = strval(  PluginLogic::get_instance()->get_rating_result( $post_ID ) );

		if ( 'hide' !== $options['results_admin'] ) :
		?><div class="mpd-align">

			<span class="mdp-stellar-rating" title="<?php esc_attr_e( 'Page Rating', 'stellar' ); ?>">
                <span class="dashicons-star-filled mdp-dashicons"></span>
                <?php esc_attr_e( $get_rating ); ?>
            </span>

			<?php if ( 'votes' === $options['results_admin'] ) : ?>
			<span class="mdp-slash">/</span>
			<span class="mdp-stellar-rating" title="<?php esc_attr_e( 'Voted', 'stellar' ); ?>">
                    <span class="dashicons-admin-users mdp-dashicons"></span>
                    <?php esc_attr_e( $get_voters ); ?>
                </span>
			<?php endif; ?>

		</div><?php
		endif;

	}

	/**
	 * Add HEAD for stellar column with results.
	 *
	 * @param array $columns
	 * @return array
	 * @since 1.0.0
	 * @access public
	 **/
	public function add_head_column( $columns ) {

		/** Add new column to the existing columns. */
		$new = [];

		/** If we have comments column, add after it. */
		$add_after = 'title';
		if ( isset( $columns['comments'] ) ) {
			$add_after = 'comments';
		} elseif ( isset( $columns['author'] ) ) {
			$add_after = 'author';
		} elseif ( isset( $columns['date'] ) ) {
			$add_after = 'date';
		}

		foreach ( $columns as $key => $col ) {

			$new[$key] = $col;

			// After Comments column.
			if ( $key === $add_after ) {
				$new['stellar-results'] = '<span class="dashicons dashicons-star-filled mdp-dashicons" title="' . esc_attr__('Rating', 'stellar' ) . '"><span class="screen-reader-text">' . esc_attr__('Rating', 'stellar' ) . '</span></span>';
			}

		}

		/** Return a new column array to WordPress. */
		return $new;
	}

	/**
	 * Register our custom column stellar-results as 'sortable'.
	 *
	 * @param array $sortable_columns - An array of sortable columns.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return mixed
	 **/
	public function sortable_result_column( $sortable_columns ) {

		$sortable_columns['stellar-results'] = 'stellar-results';

		return $sortable_columns;

	}

	/**
	 * Main AdminPostTable Instance.
	 * Insures that only one instance of AdminPostTable exists in memory at any one time.
	 *
	 * @static
	 * @since 1.0.0
	 * @access public
	 *
	 * @return AdminPostTable
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
