<?php
/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

use stdClass;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

final class Ajax {

	/**
	 * The one true Ajax.
	 *
	 * @var Ajax
	 **/
	private static $instance;

	/**
	 * Sets up a new Ajax instance.
	 *
	 * @access public
	 **/
	private function __construct() {

		if ( defined('DOING_AJAX') ) {
			add_action( 'wp_ajax_import_rating_form', [ $this, 'import_rating_form' ] );
		}

	}

	/**
	 * Handler for the action 'stellar_get_rating'.
	 * Get rating for single stellar id.
	 *
	 * @return void
	 */
	public function get_rating() {

		check_ajax_referer( 'stellar_get_rating', 'nonce' );

		if ( ! isset( $_GET['stellar_id'] ) ) { return; }
		$stellar_id = intval( $_GET['stellar_id'] );

		$rating = PluginLogic::get_instance()->get_rating_result( $stellar_id );
		$votes = PluginLogic::get_instance()->number_of_voters( $stellar_id );

		echo json_encode(
			array(
				'votes' => esc_attr( $votes ),
				'rating' => esc_attr( $rating )
			)
		);
		wp_die();

	}

	/**
	 * Import rating values from rating form
	 * @return void
	 * @noinspection SqlNoDataSourceInspection
	 */
	public function import_rating_form()
	{

		global $wpdb;

		/** Check nonce for security. */
		check_ajax_referer( 'reset_stellar', 'nonce' );

		/** Get Rating Form rating results */
		/** @noinspection SqlDialectInspection */
		$sql = "SELECT * FROM " . $wpdb->prefix . "rating_form_rated";
		$res = $wpdb->get_results(
			$wpdb->prepare( $sql )
		);

		/** Process Rating Form results */
		$copy_counter = 0;
		$status = false;
		if ( is_array( $res ) && count( $res ) > 0 ) {

			/** Remove stellar data from table. */
			$this->truncate_stellar();

			/** Mark time when we make last reset. */
			update_option( 'mdp_stellar_reset_timestamp', date('U' ) );

			/** Save rating */
			foreach ( $res as $row ) {

				$insert_result = PluginLogic::get_instance()->insert_stellar(
					$row->post_id,
					$row->rated,
					5,
					$this->generate_ip(),
					$this->generate_guid(),
					$row->date,
					$row->date
				);

				if ( $insert_result ) {
					$copy_counter++;
				}

			}
			$status = true;

		}

		/** Return JSON result. */
		echo json_encode (
			array(
				'status'    => $status,
				'counter'   => $copy_counter
			)
		);

		wp_die();
	}

	/**
	 * Remove stellar data from table
	 * @return array|object|stdClass[]|null
	 */
	public function truncate_stellar()
	{
		global $wpdb;

		$sql = "TRUNCATE TABLE " . $wpdb->prefix . "stellar";
		return $wpdb->get_results(
			$wpdb->prepare( $sql )
		);
	}

	/**
	 * Generate GUID
	 * @return string
	 */
	private function generate_guid(): string {

		return wp_sprintf(
			'%s-%s-%s-%s-%s',
			$this->random_string( 8 ),
			$this->random_string( 4 ),
			$this->random_string( 4 ),
			$this->random_string( 4 ),
			$this->random_string( 12 )
		);

	}

	/**
	 * Generate IP
	 * @return string
	 */
	private function generate_ip(): string {

		return wp_sprintf(
			'%s.%s.%s.%s',
			$this->random_string( 3, true ),
			$this->random_string( 3, true ),
			$this->random_string( 3, true ),
			$this->random_string( 3, true )
		);

	}

	/**
	 * Generate random string
	 *
	 * @param int $length
	 * @param bool $only_nums
	 *
	 * @return string
	 */
	private function random_string( int $length = 10, bool $only_nums = false ): string {

		$characters = $only_nums ? '0123456789' : '0123456789abcdefghijklmnopqrstuvwxyz';
		$charactersLength = strlen($characters);
		$randomString = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[rand(0, $charactersLength - 1)];
		}
		return $randomString;

	}

	/**
	 * Main Ajax Instance.
	 *
	 * @return Ajax
	 **/
	public static function get_instance(): Ajax {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
