<?php
/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

use Merkulove\Stellar\Unity\Plugin;
use Merkulove\Stellar\Unity\Settings;
use Merkulove\Stellar\Unity\TabGeneral;
use Merkulove\Stellar\Unity\Helper;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Settings class used to modify default plugin settings.
 *
 * @since 1.0.0
 **/
final class Config {

	/**
	 * The one true Settings.
	 *
     * @since 1.0.0
     * @access private
	 * @var Config
	 **/
	private static $instance;

    /**
     * Prepare plugin settings by modifying the default one.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function prepare_settings() {

        /** Get default plugin settings. */
        $tabs = Plugin::get_tabs();

        /** Short hand access to plugin settings. */
        $options = Settings::get_instance()->options;

        /** General Tab */

        $tabs[ 'general' ][ 'fields' ][ 'cpt_support' ] = [
            'type'              => 'cpt',
            'label'             => esc_html__( 'Post types:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Select post types for which the plugin will work', 'stellar' ),
            'show_description'  => true,
            'default'           => [ 'post', 'page' ],
            'options'           => Helper::get_instance()->get_cpt( ['exclude' => ['attachment', 'elementor_library'] ] ),
            'attr'              => [
                'multiple' => 'multiple',
            ]

        ];

        $tabs[ 'general' ][ 'fields' ][ 'position' ] = [
            'type' => 'select',
            'label' => esc_html__( 'Position:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Position', 'stellar' ),
            'description'       => esc_html__( 'Select a place on the page to display Stellar.', 'stellar' ),
            'show_description'  => true,
            'default'           => 'after-content',
            'options'           => [
                'before-title' => esc_html__( 'Before Title', 'stellar' ),
                'after-title' => esc_html__( 'After Title', 'stellar' ),
                'before-content' => esc_html__( 'Before Content', 'stellar' ),
                'after-content' => esc_html__( 'After Content', 'stellar' ),
                'shortcode' => esc_html__( 'Shortcode [stellar]', 'stellar' ),
            ]
        ];

        $key = 'number_icons';
        $default = 3;
        $tabs[ 'general' ][ 'fields' ][ $key ] = [
            'type'              => 'slider',
            'label'             => esc_html__( 'Rating points:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Number of rating icons:', 'stellar' ) .
                ' <strong>' .
                esc_html( ( isset( $options[$key] ) ) ? $options[$key] : $default ) .
                '</strong>',
            'show_description'  => true,
            'min'               => 1,
            'max'               => 12,
            'step'              => 1,
            'default'           => $default,
            'discrete'          => true,
        ];

        $tabs[ 'general' ][ 'fields' ][ 'icon' ] = [
            'type'              => 'icon',
            'label'             => esc_html__( 'Select icon:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => '',
            'description'       => esc_html__( 'SVG icon for rating', 'stellar' ),
            'show_description'  => true,
            'default'           => '_stellar/star-bold.svg',
            'meta'              => [
                '_stellar.json',
                'font-awesome.json',
                'material.json'
            ]
        ];

        $key = 'icon_size';
        $default = 24;
        $tabs[ 'general' ][ 'fields' ][ $key ] = [
            'type'              => 'slider',
            'label'             => esc_html__( 'Icon size:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'The current size of the icons:', 'stellar' ) .
                ' <strong>' .
                esc_html( ( isset( $options[$key] ) ) ? $options[$key] : $default ) .
                '</strong>' .
                esc_html__( ' px', 'stellar' ),
            'show_description'  => true,
            'min'               => 10,
            'max'               => 60,
            'step'              => 1,
            'default'           => $default,
            'discrete'          => true,
        ];

        $key = 'icon_space';
        $default = 3;
        $tabs[ 'general' ][ 'fields' ][ $key ] = [
            'type'              => 'slider',
            'label'             => esc_html__( 'Icon space:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Space between icons:', 'stellar' ) .
                ' <strong>' .
                esc_html( ( isset( $options[$key] ) ) ? $options[$key] : $default ) .
                '</strong>' .
                esc_html__( ' px', 'stellar' ),
            'show_description'  => true,
            'min'               => 0,
            'max'               => 50,
            'step'              => 1,
            'default'           => $default,
            'discrete'          => true,
        ];

        $tabs[ 'general' ][ 'fields' ][ 'inactive_icon_color' ] = [
            'type'              => 'colorpicker',
            'label'             => esc_html__( 'Default icon color:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Icon color', 'stellar' ),
            'description'       => esc_html__( 'Select the icon color', 'stellar' ),
            'show_description'  => true,
            'default'           => 'rgba(214, 213, 207, 1)',
            'attr'              => [
                'readonly'      => 'readonly',
            ]
        ];

        $tabs[ 'general' ][ 'fields' ][ 'active_icon_color' ] = [
            'type'              => 'colorpicker',
            'label'             => esc_html__( 'Active icon color:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Icon color', 'stellar' ),
            'description'       => esc_html__( 'Select the active icon color', 'stellar' ),
            'show_description'  => true,
            'default'           => 'rgba(252, 204, 0, 1)',
            'attr'              => [
                'readonly'      => 'readonly',
            ]
        ];

        $tabs[ 'general' ][ 'fields' ][ 'hover_icon_color' ] = [
            'type'              => 'colorpicker',
            'label'             => esc_html__( 'Hover icon color:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Icon color', 'stellar' ),
            'description'       => esc_html__( 'Select the hover icon color', 'stellar' ),
            'show_description'  => true,
            'default'           => 'rgba(246, 222, 0, 1)',
            'attr'              => [
                'readonly'      => 'readonly',
            ]
        ];

        $tabs[ 'general' ][ 'fields' ][ 'divider_layouts' ] = [ 'type' => 'divider',
            'label'             => '',
            'show_label'        => false,
            'default'           => '',
        ];

        $tabs[ 'general' ][ 'fields' ][ 'layout' ] = [
            'type' => 'layout',
            'label' => esc_html__( 'Layout:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Layout', 'stellar' ),
            'description'       => esc_html__( 'Select a layout for buttons and description', 'stellar' ),
            'show_description'  => true,
            'default'           => 'row-right',
            'options'           => [
                'bottom-left' => esc_html__( 'Bottom Left', 'stellar'),
                'bottom-center' => esc_html__( 'Bottom Center', 'stellar'),
                'bottom-right' => esc_html__( 'Bottom Right', 'stellar'),
                'row-left' => esc_html__( 'Row Left', 'stellar'),
                'row-right' => esc_html__( 'Row Right', 'stellar'),
                'top-center' => esc_html__( 'Top Center', 'stellar'),
                'top-left' => esc_html__( 'Top Left', 'stellar'),
                'top-right' => esc_html__( 'Top Right', 'stellar')
            ]
        ];

        $tabs[ 'general' ][ 'fields' ][ 'description' ] = [
            'type'              => 'editor',
            'label'             => esc_html__( 'Description', 'stellar' ),
            'show_label'        => true,
            'description'       => '',
            'show_description'  => false,
            'default'           => '<h4>' . esc_html__( 'Please rate Stellar', 'stellar' ) . '</h4>',
            'attr'              => [
                'textarea_rows' => '3',
            ]
        ];

        /** Results Tab */

        $offset = 1;
        $tabs = array_slice( $tabs, 0, $offset, true ) +
            ['backend' => [
                'enabled'       => true,
                'class'         => TabGeneral::class, // Handler
                'label'         => esc_html__( 'Results', 'stellar' ),
                'title'         => esc_html__( 'Results Display Settings', 'stellar' ),
                'show_title'    => true,
                'icon'          => 'favorite', // Icon for tab
                'fields'        => []
            ] ] +
            array_slice( $tabs, $offset, NULL, true );
        $backend_options = is_array( get_option( 'mdp_stellar_backend_settings' ) );

        $tabs[ 'backend' ][ 'fields' ][ 'latest' ] = [ 'type' => 'select',
            'label' => esc_html__( 'Current user\'s rating:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'User voting', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide user latest voting results', 'stellar' ),
            'show_description'  => true,
            'default'           => 'hide',
            'options'           => [
                'hide'          => esc_html__( 'Hide user voting', 'stellar' ),
                'stars'         => esc_html__( 'Show as stars', 'stellar' ),
                'text'          => esc_html__( 'Show as text', 'stellar' ),
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'latest_text' ] = [ 'type' => 'text',
            'label'             => esc_html__( 'User vote text:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Results', 'stellar' ),
            'description'       => esc_html__( 'This message will be shown for user votes results', 'stellar' ),
            'show_description'  => true,
            'default'           => esc_html__( 'Your page rank:', 'stellar' ),
            'attr'              => [
                'maxlength' => '300'
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'results' ] = [ 'type' => 'select',
            'label' => esc_html__( 'Results on front-end:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Results', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide results on the front-end', 'stellar' ),
            'show_description'  => true,
            'default'           => 'hide',
            'options'           => [
                'hide'              => esc_html__( 'Hide results', 'stellar' ),
                'before'            => esc_html__( 'Before rating icons', 'stellar' ),
                'after'             => esc_html__( 'After rating icons', 'stellar' ),
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'results_text' ] = [ 'type' => 'text',
            'label'             => esc_html__( 'Results text:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Results', 'stellar' ),
            'description'       => esc_html__( 'This message will be shown next to the voting results', 'stellar' ),
            'show_description'  => true,
            'default'           => '',
            'attr'              => [
                'maxlength' => '300'
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'votes' ] = [ 'type' => 'select',
            'label' => esc_html__( 'Total voters on front-end:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Votes', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide the number of voters on each page', 'stellar' ),
            'show_description'  => true,
            'default'           => 'hide',
            'options'           => [
                'hide'              => esc_html__( 'Hide votes count', 'stellar' ),
                'before'            => esc_html__( 'Before rating icons', 'stellar' ),
                'after'             => esc_html__( 'After rating icons', 'stellar' ),
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'votes_text' ] = [ 'type' => 'text',
            'label'             => esc_html__( 'Votes count text:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Votes', 'stellar' ),
            'description'       => esc_html__( 'This message will be shown next to the number of votes', 'stellar' ),
            'show_description'  => true,
            'default'           => '',
            'attr'              => [
                'maxlength' => '300'
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'valuation' ] = [ 'type' => 'select',
            'label' => esc_html__( 'Valuation on front-end:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Valuation', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide user valuation on the front-end', 'stellar' ),
            'show_description'  => true,
            'default'           => 'tooltip',
            'options'           => [
                'hide'              => esc_html__( 'Hide valuation', 'stellar' ),
                'tooltip'           => esc_html__( 'Show as tooltip', 'stellar' ),
                'val-before'        => esc_html__( 'Show before rating icons', 'stellar' ),
                'val-after'         => esc_html__( 'Show after rating icons', 'stellar' ),
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'round' ] = [ 'type' => 'select',
            'label' => esc_html__( 'Rounding for icons:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Rounding', 'stellar' ),
            'description'       => esc_html__( 'Rounding the rating to fill icons with color', 'stellar' ),
            'show_description'  => true,
            'default'           => 'up',
            'options'           => [
                'up'            => esc_html__( 'Up', 'stellar' ),
                'no'            => esc_html__( 'Do not round', 'stellar' ),
                'down'          => esc_html__( 'Down', 'stellar' ),
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'secrecy' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Display results before voting', 'stellar' ) . ':',
            'placeholder'       => esc_html__( 'Display results', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide results before voting', 'stellar' ),
            'default'           => 'off',
        ];

	    $tabs[ 'backend' ][ 'fields' ][ 'refresh' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Refresh rating after page load', 'stellar' ) . ':',
		    'placeholder'       => esc_html__( 'Refresh', 'stellar' ),
		    'description'       => esc_html__( 'Refresh data after each page load to avoid caching results on the page.', 'stellar' ),
		    'default'           => 'off',
	    ];

        $tabs[ 'backend' ][ 'fields' ][ 'divider_limit' ] = [ 'type' => 'divider',
            'label'             => '',
            'show_label'        => false,
            'default'           => '',
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'session' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Session control:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Track sessions', 'stellar' ),
            'description'       => esc_html__( 'Allows/Deny re-voting after the session ends', 'stellar' ),
            'show_description'  => true,
            'default'           => 'off',
        ];

        $key = 'limit_voting';
        $default = 1;
        $tabs[ 'backend' ][ 'fields' ][$key] = [
            'type'              => 'slider',
            'label'             => esc_html__( 'Voting limit:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'The limit of votes from one IP:', 'stellar' ) .
                ' <strong>' .
                esc_html( ( $backend_options ) ? get_option( 'mdp_stellar_backend_settings' )[ $key ] : $default ) .
                '</strong>',
            'show_description'  => true,
            'min'               => 1,
            'max'               => 10,
            'step'              => 1,
            'default'           => $default,
            'discrete'          => true,
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'limit_text' ] = [ 'type' => 'text',
            'label'             => esc_html__( 'Limit exceeded message:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Message', 'stellar' ),
            'description'       => esc_html__( 'This message will be shown when the user exceeds the vote limit.', 'stellar' ),
            'show_description'  => true,
            'default'           => esc_html__( 'You have already voted. Sorry.', 'stellar' ),
            'attr'              => [
                'maxlength' => '300'
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'divider_reset' ] = [ 'type' => 'divider',
            'label'             => '',
            'show_label'        => false,
            'default'           => '',
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'results_admin' ] = [ 'type' => 'select',
            'label' => esc_html__( 'Results in the admin area:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Results', 'stellar' ),
            'description'       => esc_html__( 'The format for displaying the results in the admin area', 'stellar' ),
            'show_description'  => true,
            'default'           => 'rating',
            'options'           => [
                'rating'    => esc_html__( 'Rating', 'stellar' ),
                'votes'     => esc_html__( 'Rating / Votes', 'stellar' ),
                'hide'      => esc_html__( 'Hide', 'stellar' ),
            ]
        ];

        $tabs[ 'backend' ][ 'fields' ][ 'reset_stellar_results' ] = [ 'type'  => 'button',
            'label'             => esc_html__( 'Reset ratings:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Reset', 'stellar' ),
            'description'       => esc_html__( 'Press to reset completely', 'stellar' ),
            'show_description'  => true,
            'default'           => '',
            'icon'              => 'close',
            'attr'              => [
                'class'     => 'mdp-reset mdc-button--unelevated',
                "id" => "reset",
            ]
        ];

		# Rating form import
		if ( is_plugin_active( 'rating-form/rf-init.php' ) ) {

			$tabs[ 'backend' ][ 'fields' ][ 'import_rating_form' ] =
				[ 'type'  => 'button',
				  'label'             => esc_html__( 'Rating Form import:', 'stellar' ),
				  'show_label'        => true,
				  'placeholder'       => esc_html__( 'Import from Rating Form', 'stellar' ),
				  'description'       => esc_html__( 'Reset saved rating data and copy data from the Rating Form plugin', 'stellar' ),
				  'show_description'  => true,
				  'default'           => '',
				  'icon'              => 'content_copy',
				  'attr'              => [
					  'class'     => 'mdp-import-rating-form mdc-button--unelevated',
					  "id" => "import-rating-form",
				]
			];

		}

        /** Rich Snippet Tab */

        $offset = 2;
        $tabs = array_slice( $tabs, 0, $offset, true ) +
            [ 'schema' => [
                'enabled'       => true,
                'class'         => TabGeneral::class, // Handler
                'label'         => esc_html__( 'Rich Snippet', 'stellar' ),
                'title'         => esc_html__( 'Rich Snippet Settings', 'stellar' ),
                'show_title'    => true,
                'icon'          => 'pageview', // Icon for tab
                'fields'        => []
            ] ] +
            array_slice( $tabs, $offset, NULL, true );

        $tabs[ 'schema' ][ 'fields' ][ 'google_search_results' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Schema Markup:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => '',
            'description'       => esc_html__( 'SEO microdata for search indexes', 'stellar' ),
            'show_description'  => true,
            'default'           => 'off',
        ];

        $tabs[ 'schema' ][ 'fields' ][ 'advanced_markup' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Advanced Schema Markup:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Advanced Schema Markup', 'stellar' ),
            'description'       => esc_html__( 'Provide the JSON+LD structure', 'stellar' ),
            'show_description'  => true,
            'default'           => 'off',
        ];

        $tabs[ 'schema' ][ 'fields' ][ 'json_ld' ] = [
            'type'              => 'textarea',
            'label'             => esc_html__( 'JSON+LD Markup:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'JSON', 'stellar' ),
            'description'       => esc_html__( 'The following variables are available: ', 'stellar' ) .
                '<br><strong>[title]</strong>' . esc_html__( ' — Post Title.', 'stellar' ) .
                '<br><strong>[value]</strong>' . esc_html__( ' — A numerical quality rating for the item.', 'stellar' ) .
                '<br><strong>[best]</strong>' . esc_html__( ' — The highest value allowed in this rating system.', 'stellar' ) .
                '<br><strong>[count]</strong>' . esc_html__( ' — The total number of ratings for the post on your site. ', 'stellar' ),
            'show_description'  => true,
            'default'           => $this->get_json_ld_default_value(),
            'attr'              => [
                'spellcheck'    => 'false',
                'wrapper-class' => 'mdp-stellar-backend-settings-json-ld',
            ]
        ];

		if ( isset( $options[ 'position' ] ) && $options[ 'position' ] === 'shortcode' ) {

			$tabs[ 'schema' ][ 'fields' ][ 'markup_everywhere' ] = [
				'type'              => 'switcher',
				'label'             => esc_html__( 'Advanced Shortcode markup:', 'stellar' ),
				'placeholder'       => esc_html__( 'Advanced Shortcode markup', 'stellar' ),
				'description'       => esc_html__( 'Show markup if post has at least one rating and the Stellar Shortcode is called on the page.', 'stellar' ),
				'default'           => 'off',
			];

		}

        $offset = 3;
        $tabs = array_slice( $tabs, 0, $offset, true ) +
            [ 'shortcode' => [
                'enabled'       => true,
                'class'         => TabGeneral::class, // Handler
                'label'         => esc_html__( 'Shortcode', 'stellar' ),
                'title'         => esc_html__( 'Shortcode settings of Top-rated posts list', 'stellar' ),
                'show_title'    => true,
                'icon'          => 'stars', // Icon for tab
                'fields'        => []
            ] ] +
            array_slice( $tabs, $offset, NULL, true );
        $shortcode_options = is_array( get_option( 'mdp_stellar_shortcode_settings' ) );

	    # Shortcode Header.
	    $tabs['shortcode']['fields']['shortcode_header'] = [
		    'type'              => 'header',
		    'label'             => '',
		    'show_label'        => false,
		    'description'       => $this->get_shortcode_description(),
		    'show_description'  => true,
		    'default'           => ''
	    ];

        $key = 'top_gutter';
        $default = 11;
        $tabs[ 'shortcode' ][ 'fields' ][ $key ] = [
            'type'              => 'slider',
            'label'             => esc_html__( 'Gutter:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Space between posts:', 'stellar' ) .
                ' <strong>' .
                esc_html( ( $shortcode_options ) ? get_option( 'mdp_stellar_shortcode_settings' )[ $key ] : $default ) .
                '</strong>' . esc_html__( 'px', 'stellar' ),
            'show_description'  => true,
            'min'               => 0,
            'max'               => 100,
            'step'              => 1,
            'default'           => $default,
            'discrete'          => true,
        ];

        $tabs[ 'shortcode' ][ 'fields' ][ 'top_image' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Post image:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Display post image', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide post image', 'stellar' ),
            'show_description'  => true,
            'default'           => 'on',
        ];

        $tabs[ 'shortcode' ][ 'fields' ][ 'top_image_size' ] = [
            'type' => 'select',
            'label' => esc_html__( 'Image size :', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Image size', 'stellar' ),
            'description'       => esc_html__( 'Select the image size for post', 'stellar' ),
            'show_description'  => true,
            'default'           => 'large',
            'options'           => [
                'thumbnail' => esc_html__( 'Thumbnail', 'stellar' ),
                'medium' => esc_html__( 'Medium', 'stellar' ),
                'large' => esc_html__( 'Large', 'stellar' ),
                'full' => esc_html__( 'Full', 'stellar' ),
            ]
        ];

        $tabs[ 'shortcode' ][ 'fields' ][ 'top_equal' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Equal image:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Equal image height', 'stellar' ),
            'description'       => esc_html__( 'Allow/Deny equal image sizes', 'stellar' ),
            'show_description'  => true,
            'default'           => 'on',
        ];

        $key = 'top_height';
        $default = 320;
        $tabs[ 'shortcode' ][ 'fields' ][ $key ] = [
            'type'              => 'slider',
            'label'             => esc_html__( 'Image height:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Image height:', 'stellar' ) .
                ' <strong>' .
                esc_html( ( $shortcode_options ) ? get_option( 'mdp_stellar_shortcode_settings' )[ $key ] : $default ) .
                '</strong>' . esc_html__( 'px', 'stellar' ),
            'show_description'  => true,
            'min'               => 0,
            'max'               => 640,
            'step'              => 1,
            'default'           => $default,
            'discrete'          => true,
        ];

        $tabs[ 'shortcode' ][ 'fields' ][ 'top_title' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Post title:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Display post title', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide post title', 'stellar' ),
            'show_description'  => true,
            'default'           => 'on',
        ];

        $tabs[ 'shortcode' ][ 'fields' ][ 'top_title_tag' ] = [
            'type' => 'select',
            'label' => esc_html__( 'Post title tag :', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Title HTML tag', 'stellar' ),
            'description'       => esc_html__( 'Select the tag for post title', 'stellar' ),
            'show_description'  => true,
            'default'           => 'h3',
            'options'           => [
                'h1' => esc_html__( 'H1', 'stellar' ),
                'h2' => esc_html__( 'H2', 'stellar' ),
                'h3' => esc_html__( 'H3', 'stellar' ),
                'h4' => esc_html__( 'H4', 'stellar' ),
                'h5' => esc_html__( 'H5', 'stellar' ),
                'h6' => esc_html__( 'H6', 'stellar' ),
            ]
        ];

        $tabs[ 'shortcode' ][ 'fields' ][ 'top_excerpt' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Post excerpt:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Display post excerpt', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide post excerpt', 'stellar' ),
            'show_description'  => true,
            'default'           => 'off',
        ];

        $tabs[ 'shortcode' ][ 'fields' ][ 'top_rating' ] = [
            'type'              => 'switcher',
            'label'             => esc_html__( 'Post rating:', 'stellar' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Display post rating', 'stellar' ),
            'description'       => esc_html__( 'Show/Hide post rating', 'stellar' ),
            'show_description'  => true,
            'default'           => 'on',
        ];

        $key = 'top_size';
        $default = 14;
        $tabs[ 'shortcode' ][ 'fields' ][ $key ] = [
            'type'              => 'slider',
            'label'             => esc_html__( 'Rating icon size:', 'stellar' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Icon size:', 'stellar' ) .
                ' <strong>' .
                esc_html( ( $shortcode_options ) ? get_option( 'mdp_stellar_shortcode_settings' )[ $key ] : $default ) .
                '</strong>' . esc_html__( 'px', 'stellar' ),
            'show_description'  => true,
            'min'               => 1,
            'max'               => 50,
            'step'              => 1,
            'default'           => $default,
            'discrete'          => true,
        ];

        /** Set updated tabs. */
        Plugin::set_tabs( $tabs );

        /** Refresh settings. */
        Settings::get_instance()->get_options();

    }

	/**
	 * Return HTML description for "shortcode_header" field.
	 *
	 * @since 2.1.1
	 *
	 * @return string
	 **/
    private function get_shortcode_description() {

    	return esc_html__( 'Add shortcode', 'stellar' ) .
	           ' <code>[stellar top="10" for="page" cols="4"]</code> ' .
	           esc_html__( 'to display the list of top-rated posts. Read more about the shortcode parameters in the ', 'stellar' ) .
	           '<a href="https://docs.merkulov.design/setting-a-star-rating-for-wordpress-pages/#Shortcode" target="_blank">' .
	           esc_html__( 'documentation', 'stellar' ) .
	           '</a>';

    }

    /**
     * Return json_ld default value.
     *
     * @since 1.1.0
     *
     * @return string
     **/
    private function get_json_ld_default_value() {

        return '{
    "@context": "https://schema.org/",
    "@type": "CreativeWorkSeries",
    "name": "[title]",
    "aggregateRating": {
        "@type": "AggregateRating",
        "ratingValue": "[value]",
        "bestRating": "[best]",
        "ratingCount": "[count]"
    }
}';

    }

	/**
	 * Main Settings Instance.
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Config
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
