<?php
/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

use Merkulove\Stellar\Unity\Plugin;
use Merkulove\Stellar\Unity\Settings;
use Merkulove\Stellar\Unity\TabAssignments;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class adds admin styles.
 *
 * @since 1.0.0
 *
 **/
final class FrontEnd {

	/**
	 * @var FrontEnd
	 * @since 1.0.0
	 **/
	private static $instance;

    /**
     * @var array
     */
	private static $options;

	/**
	 * Sets up a new FrontStyles instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

        /** Load JS and CSS for Frontend Area. */
	    $this->enqueue_frontend();

	}

	/**
	 * Add plugin styles.
	 *
	 * @since 1.0.0
	 * @return void
	 **/
	public function enqueue_frontend() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

	    self::$options = Settings::get_instance()->options;

        /** Add plugin styles. */
        add_action( 'wp_enqueue_scripts', [ $this, 'load_styles' ] );
        add_action( 'wp_enqueue_scripts', [ $this, 'style_icon' ] );

        /** Add plugin script. */
        add_action( 'wp_enqueue_scripts', [ $this, 'load_scripts' ] );

	}

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since   1.0.0
     * @return void
     **/
    public function load_scripts( $ignore_assignments = false ) {

        /** Checks if plugin should work on this page. */
        if ( ! $ignore_assignments ) {
            if ( !TabAssignments::get_instance()->display() ) {
                return;
            }
        }

        /** Connect custom JS library. */
        wp_enqueue_script( 'mdp-stellar', Plugin::get_url() . 'js/stellar' . Plugin::get_suffix() . '.js', [], Plugin::get_version(), true );

        /** In JavaScript, object properties are accessed as mdpStellarAJAX.url */
        wp_localize_script( 'mdp-stellar', 'mdpStellar', [
            'url' => admin_url( 'admin-ajax.php' ),
            'nonce' => array(
				'getRating' => wp_create_nonce( 'stellar_get_rating' ),
            ),
            'reset_timestamp' => get_option( 'mdp_stellar_reset_timestamp' ),
            'length' => self::$options['number_icons'],
            'results_text' => self::$options['results_text'],
            'limitText' => self::$options['limit_text'],
            'session' => self::$options['session']
        ] );

    }

    /**
     * Add CSS for the public-facing side of the site.
     *
     * @param bool $ignore_assignments
     * @return void
     * @since   1.0.0
     */
    public function load_styles( $ignore_assignments = false ) {

        /** Checks if plugin should work on this page. */
        if ( ! $ignore_assignments ) {
            if ( !TabAssignments::get_instance()->display() ) {
                return;
            }
        }

        wp_enqueue_style( 'mdp-stellar', Plugin::get_url() . 'css/stellar' . Plugin::get_suffix() . '.css', [], Plugin::get_version() );

        $custom_css = self::$options['custom_css'];

        wp_add_inline_style( 'mdp-stellar', $custom_css );

    }

    /**
     * Stellar inline CSS variables
     */
    public function style_icon() {

        $size = self::$options['icon_size'];
        $color = self::$options['inactive_icon_color'];
        $active_color = self::$options['active_icon_color'];
        $hover_color = self::$options['hover_icon_color'];
        $icon_space = self::$options['icon_space'];

        $top_size = self::$options['top_size'];
        $top_gutter = self::$options['top_gutter'];

        $css = "
        .mdp-stellar-box {
            --mdp-stellar-size: {$size}px;
            --mdp-stellar-color: {$color};
            --mdp-stellar-active: {$active_color};
            --mdp-stellar-hover: {$hover_color};
            --mdp-stellar-space: {$icon_space}px;
        }
        .mdp-stellar-top {
            --mdp-stellar-size: {$top_size}px;
            --mdp-stellar-color: {$color};
            --mdp-stellar-active: {$active_color};
            --mdp-stellar-hover: {$hover_color};
            --mdp-stellar-space: {$icon_space}px;
            --mdp-stellar-gutter: {$top_gutter}px;
        }";

        wp_add_inline_style( 'mdp-stellar', $css );

    }

	/**
	 * Main FrontStyles Instance.
	 *
	 * Insures that only one instance of FrontStyles exists in memory at any one time.
	 *
	 * @static
	 * @return FrontEnd
	 * @since 1.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
