<?php
/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

use Merkulove\Stellar\Unity\Settings;
use Merkulove\Stellar\Unity\TabAssignments;

/**
 * SINGLETON: Class adds admin styles.
 *
 * @since 1.0.0
 *
 **/
final class Shortcode {

	/**
	 * @var Shortcode
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new FrontStyles instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

        /** Add stellar top shortcode */
        add_shortcode( 'stellar', [ $this, 'stellar_shortcode' ] );
        add_shortcode( 'stellar-rating', [ $this, 'stellar_rating_shortcode' ] );
        add_shortcode( 'stellar-count', [ $this, 'stellar_count_shortcode' ] );

        /** Add styles and scripts for shortcodes */
        add_action( 'mdp_stellar_shortcode_css', [ FrontEnd::get_instance(), 'load_styles' ], 11, 1 );
        add_action( 'mdp_stellar_shortcode_js', [ FrontEnd::get_instance(), 'load_scripts' ], 11, 1 );
        add_action( 'mdp_stellar_view', [ FrontEnd::get_instance(), 'style_icon' ], 12 );

	}

    /**
     * Add stellar by short code [stellar].
     * Options add shortcode with option to show top rated posts
     *
     * @param $atts
     *
     * @since  1.0.0
     * @access public
     *
     * @return string - result short code.
     */
    public function stellar_shortcode( $atts ) {

        /** Load styles and scripts for shortcode */
        if ( isset( $atts[ 'top' ] ) || isset( $atts[ 'for' ] ) ) {

	        do_action( 'mdp_stellar_shortcode_css', true );
	        do_action( 'mdp_stellar_shortcode_js', true );
	        do_action( 'mdp_stellar_view' );

        }

        /** Render Shortcode */
        if ( isset( $atts[ 'top' ] ) ) { // Stellar Top Rated posts

	        return $this->shortcode_stellar_top( $atts );

        } else if ( isset( $atts[ 'average' ] ) ) { // Average rating by post type

            return $this->shortcode_stellar_average( $atts );

        } else if ( isset( $atts[ 'overall' ] ) ) { // Stellar overall website rating

	        return $this->shortcode_stellar_overall();

        } else if ( Settings::get_instance()->options[ 'position' ] === 'shortcode' ) {

            if ( ! TabAssignments::get_instance()->display() ) { return ''; } //Checks if plugin should work on this page

            return PluginLogic::get_instance()->get_stellar( $atts ); // Stellar

        } else {

	        return '';

        }

    }

	/**
     * Shortcode [stellar-rating] for displaying rating value
	 *
	 * @param $atts
	 *
	 * @return bool
	 */
    public function stellar_rating_shortcode( $atts ) {

	    $id = isset( $atts[ 'id' ] ) ? (int) $atts['id'] : get_the_ID();

	    ob_start();
        echo esc_attr( PluginLogic::get_instance()->get_rating_result( $id ) );
	    return ob_get_clean();

    }

	/**
     * Shortcode [stellar-count] for displaying rating count
	 *
	 * @param $atts
	 *
	 * @return bool
	 */
	public function stellar_count_shortcode( $atts ) {

		$id = isset( $atts[ 'id' ] ) ? (int) $atts['id'] : get_the_ID();

		ob_start();
		echo esc_attr( PluginLogic::get_instance()->number_of_voters( $id ) );
		return ob_get_clean();

	}

    /**
     * Add stellar top posts by short code [stellar top="10" for="page" cols="4"].
     *
     * @param $atts - shortcode attributes
     *
     * @return string
     **/
    public function shortcode_stellar_top( $atts ) {

        global $wpdb;

        // Get shortcode attributes
        $top = isset( $atts[ 'top' ] ) ? (int) $atts['top'] : 4;
        $for = isset( $atts['for'] ) ? $atts['for'] : 'post';
        $cols = isset( $atts[ 'cols' ] ) ? (int) $atts['cols'] : 1;

        // Get plugin settings
        $options = Settings::get_instance()->options;

        // Get options
        $image = $options[ 'top_image' ] === 'on';
        $title = $options[ 'top_title' ] === 'on';
        $excerpt = $options[ 'top_excerpt' ] === 'on';
        $rating = $options[ 'top_rating' ] === 'on';
        $equal = $options[ 'top_equal' ] === 'on';
        $height = $options[ 'top_height' ] . 'px';
        $image_size = $options[ 'top_image_size' ];
        $tag = $options[ 'top_title_tag' ];

        // Get top liked 5 posts of selected $post_type
        /** @noinspection SqlNoDataSourceInspection */
        $posts = $wpdb->get_results(
            $wpdb->prepare("
                SELECT $wpdb->stellar.stellar_id as post_id, ROUND( SUM( $wpdb->stellar.rating / $wpdb->stellar.length ) * %d ) AS rating, COUNT( $wpdb->stellar.stellar_id ) as total_votes
                FROM $wpdb->stellar
                INNER JOIN $wpdb->posts ON $wpdb->stellar.stellar_id = $wpdb->posts.ID
                WHERE $wpdb->posts.post_type = %s
                GROUP BY $wpdb->stellar.stellar_id
                ORDER BY rating DESC
                LIMIT %d;", [ $options[ 'number_icons' ], $for, 1000 ] )
        );

        // No records with Stellar. Nothing to show
        if ( empty( $posts ) ) { return ''; }

        // Calculate and add page rating to array
        foreach ( $posts as $iValue ) {

            $this_post_rating = PluginLogic::get_instance()->get_rating_result( $iValue->post_id );
            $this_post_id = $iValue->post_id;

            if ( $this_post_rating > 0 ) {

                $posts_order[ $this_post_id ] = $this_post_rating;

            }

        }

        arsort( $posts_order, SORT_NUMERIC ); // Sort by rating
        $posts_order = array_slice( $posts_order, 0, $top, true ); // Crop by top value

        ob_start();

        ?>
        <!-- Stellar Top Rated: Start !-->
    <div class="mdp-stellar-top mdp-stellar-cols-<?php esc_attr_e( $cols ); ?>">
        <?php foreach ( $posts_order as $post_id => $post_rating ) : ?>
            <div class="mdp-stellar-post">
                <?php if ( $image ) :

                    if ( $equal ) { ?>
                        <a class="mdp-stellar-post-image"
                           style="height: <?php esc_attr_e( $height ); ?>; background-image: url( <?php echo get_the_post_thumbnail_url( $post_id, $image_size ); ?> )"
                           href="<?php echo get_permalink( $post_id ); ?>"
                           title="<?php echo get_the_title( $post_id ); ?>">
                        </a>
                    <?php } else { ?>
                        <div class="mdp-stellar-post-image">
                            <a href="<?php echo get_permalink( $post_id ); ?>" title="<?php echo get_the_title( $post_id ); ?>">
                                <?php echo get_the_post_thumbnail( $post_id, $image_size ); ?>
                            </a>
                        </div>
                    <?php } ?>

                <?php endif; ?>

                <div class="mdp-stellar-post-content">
                    <?php if ( $title ) : ?>
                    <<?php esc_attr_e( $tag ); ?> class="mdp-stellar-post-title">
                    <a href="<?php echo get_permalink( $post_id ); ?>" title="<?php echo get_the_title( $post_id ); ?>">
                        <?php echo get_the_title( $post_id ); ?>
                    </a>
                </<?php esc_attr_e( $tag ); ?>>
            <?php endif; ?>

                <?php if ( $excerpt ) :?>
                    <p class="mdp-stellar-post-excerpt">
                        <?php echo get_the_excerpt( $post_id ) ?>
                    </p>
                <?php endif; ?>

                <?php if ( $rating ) : ?>
                    <div class="mdp-stellar-post-rating">
                        <div class="mdp-stellar-post-stars">
                            <?php for ( $i = 1; $i <= $options[ 'number_icons' ]; $i++ ):

                                /** Calculate segment width for icons. */

                                /** Full filled icon. */
                                if ( $i <= $post_rating ) {

                                    $percent = '100%';

                                }

                                /** 0 filled icon. */
                                elseif ( $i > $post_rating + 1 ) {

                                    $percent = '0';

                                }

                                /** Partially filled icon. */
                                else {

                                    $percent = ( fmod( $post_rating, 1 ) * 100 ) . '%';

                                }

                                ?>
                                <i data-id="<?php echo esc_attr($i); ?>">
                                    <?php echo PluginLogic::get_instance()->svg(); ?>
                                    <div class="mdp-stellar-segment" style="width: <?php esc_attr_e( $percent ); ?>"><?php echo PluginLogic::get_instance()->svg() ?></div>
                                </i>
                            <?php endfor ?>
                        </div>
                        <div class="mdp-stellar-post-value">
                            <?php esc_html_e( PluginLogic::get_instance()->get_rating_result( $post_id ) ); ?>
                        </div>
                    </div>
                <?php endif; ?>
            </div>
            </div>
        <?php endforeach; ?>
        </div>
        <!-- Stellar Top Rated: End !-->
        <?php

        return ob_get_clean();

    }

	/**
     * Overall website rating
	 * @return false|string|void
	 */
    public function shortcode_stellar_overall() {

	    global $wpdb;

	    /** Get plugin settings. */
	    $options = Settings::get_instance()->options;
	    $posts_order = array();

	    $posts = $wpdb->get_results(
		    $wpdb->prepare("
                SELECT $wpdb->stellar.stellar_id as post_id, ROUND( SUM( $wpdb->stellar.rating / $wpdb->stellar.length ) * %d ) AS rating, COUNT( $wpdb->stellar.stellar_id ) as total_votes
                FROM $wpdb->stellar
                INNER JOIN $wpdb->posts ON $wpdb->stellar.stellar_id = $wpdb->posts.ID
                GROUP BY $wpdb->stellar.stellar_id
                ORDER BY rating DESC
                LIMIT 1000;", [ $options['number_icons'] ] )
	    );

        /** Calculate overall rating */
        if ( ! empty( $posts ) ) {

	        $posts_order = PluginLogic::get_instance()->calculate_rating( $posts ); // Calculate and add page rating to array
	        $overall = number_format(array_sum( $posts_order )/count( $posts_order ), 2 );

        } else {

            $overall = 0;

        }

	    ob_start();

        esc_html_e( 'Overall Website Rating: ', 'stellar' );
	    ?><span class="mdp-stellar-overall"><?php echo esc_attr( $overall ); ?></span><?php

	    return ob_get_clean();

    }

	/**
     * Average rating by Post Type
     *
	 * @param $atts
	 *
	 * @return false|string
	 */
    public function shortcode_stellar_average( $atts ) {

	    $post_type = get_post_type_object( $atts[ 'average' ] );
	    if ( null === $post_type ) { return ''; }

	    global $wpdb;

	    /** Get plugin settings. */
	    $options = Settings::get_instance()->options;
	    $posts_order = array();

	    $posts = $wpdb->get_results(
		    $wpdb->prepare("
                SELECT $wpdb->stellar.stellar_id as post_id, ROUND( SUM( $wpdb->stellar.rating / $wpdb->stellar.length ) * %d ) AS rating, COUNT( $wpdb->stellar.stellar_id ) as total_votes
                FROM $wpdb->stellar
                INNER JOIN $wpdb->posts ON $wpdb->stellar.stellar_id = $wpdb->posts.ID
                WHERE $wpdb->posts.post_type = %s
                GROUP BY $wpdb->stellar.stellar_id
                ORDER BY rating DESC
                LIMIT 1000;", [ $options['number_icons'], $atts[ 'average' ] ] )
	    );

	    /** Calculate average rating */
	    if ( ! empty( $posts ) ) {

		    $posts_order = PluginLogic::get_instance()->calculate_rating( $posts ); // Calculate and add page rating to array
		    $average = number_format(array_sum( $posts_order )/count( $posts_order ), 2 );

	    } else {

		    $average = 0;

	    }

	    ob_start();

	    esc_html_e( 'Average ', 'stellar' );
	    esc_html_e( $post_type->label );
	    esc_html_e( ' Rating: ', 'stellar' );
        ?><span class="mdp-stellar-average"><?php echo esc_attr( $average );?></span><?php

	    return ob_get_clean();

    }

	/**
	 * Main FrontStyles Instance.
	 *
	 * Insures that only one instance of FrontStyles exists in memory at any one time.
	 *
	 * @static
	 * @return Shortcode
	 * @since 1.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
